From 52af9a5a85aadb305035139fab2a962abb4eead0 Mon Sep 17 00:00:00 2001
From: Andrii Kolomoiets <andreyk.mad@gmail.com>
Date: Sat, 23 Nov 2019 18:43:47 +0200
Subject: [PATCH] vc-hg: prompt for branch to merge

* lisp/vc/vc-hg.el (vc-hg-merge-branch): Prompt for branch to merge.
(vc-hg-revision-table): Use branches and tags as competion candidates.

* etc/NEWS Mention changes of vc-hg.el

This fixes bug#22860

diff --git a/etc/NEWS b/etc/NEWS
index ad349b1613..abd2c616bd 100644
--- a/etc/NEWS
+++ b/etc/NEWS
@@ -933,6 +933,11 @@ values.
 The 'C-x v h' command now works in buffers that visit files controlled
 by Hg.

+*** The Hg (Mercurial) back-end now prompt for branch to merge.
+
+*** The Hg (Mercurial) back-end now use tag and branch names instead of
+revision numbers as completion candidates on reading revision.
+
 +++
 *** 'C-u C-x v D' ('vc-root-version-diff') prompts for two revisions
 and compares their entire trees.
diff --git a/lisp/vc/vc-hg.el b/lisp/vc/vc-hg.el
index 17d38fa400..b6cb315c1b 100644
--- a/lisp/vc/vc-hg.el
+++ b/lisp/vc/vc-hg.el
@@ -564,7 +564,8 @@ vc-hg-expanded-log-entry
 (defun vc-hg-revision-table (files)
   (let ((default-directory (file-name-directory (car files))))
     (with-temp-buffer
-      (vc-hg-command t nil files "log" "--template" "{rev} ")
+      (vc-hg-command t nil nil "branches" "-q")
+      (vc-hg-command t nil nil "tags" "-q")
       (split-string
        (buffer-substring-no-properties (point-min) (point-max))))))

@@ -1483,12 +1484,15 @@ vc-hg-push
   (vc-hg--pushpull "push" prompt nil (called-interactively-p 'interactive)))

 (defun vc-hg-merge-branch ()
-  "Merge incoming changes into the current working directory.
+  "Prompt for branch and merge it into the current branch.
 This runs the command \"hg merge\"."
   (let* ((root (vc-hg-root default-directory))
 	 (buffer (format "*vc-hg : %s*" (expand-file-name root)))
          ;; Disable pager.
-         (process-environment (cons "HGPLAIN=1" process-environment)))
-    (apply 'vc-do-async-command buffer root vc-hg-program '("--config" "ui.report_untrusted=0" "merge"))
+         (process-environment (cons "HGPLAIN=1" process-environment))
+         (branch (vc-read-revision "Merge from branch: ")))
+    (apply 'vc-do-async-command buffer root vc-hg-program
+           (append '("--config" "ui.report_untrusted=0" "merge")
+                   (unless (string= branch "") (list branch))))
     (with-current-buffer buffer (vc-run-delayed (vc-compilation-mode 'hg)))
     (vc-set-async-update buffer)))

--
2.15.1
